@@ -33,6 +33,7 @@ from localstack.services.awslambda.lambda_utils import (
     ClientError,
     error_response,
     event_source_arn_matches,
+    get_executor_mode,
     get_handler_file_from_name,
     get_lambda_runtime,
     get_zip_bytes,
@@ -71,8 +72,9 @@ from localstack.utils.common import (
     unzip,
 )
 from localstack.utils.docker_utils import DOCKER_CLIENT
+from localstack.utils.functions import run_safe
 from localstack.utils.generic.singleton_utils import SubtypesInstanceManager
-from localstack.utils.http_utils import canonicalize_headers, parse_chunked_data
+from localstack.utils.http import canonicalize_headers, parse_chunked_data
 from localstack.utils.run import FuncThread
 
 # logger
@@ -129,7 +131,7 @@ JSON_START_CHARS = tuple(set(functools.reduce(lambda x, y: x + y, JSON_START_CHA
 
 # lambda executor instance
 LAMBDA_EXECUTOR = lambda_executors.AVAILABLE_EXECUTORS.get(
-    config.LAMBDA_EXECUTOR, lambda_executors.DEFAULT_EXECUTOR
+    get_executor_mode(), lambda_executors.DEFAULT_EXECUTOR
 )
 
 # IAM policy constants
@@ -502,7 +504,7 @@ def use_docker():
     global DO_USE_DOCKER
     if DO_USE_DOCKER is None:
         DO_USE_DOCKER = False
-        if "docker" in config.LAMBDA_EXECUTOR:
+        if "docker" in get_executor_mode():
             has_docker = DOCKER_CLIENT.has_docker()
             if not has_docker:
                 LOG.warning(
@@ -511,7 +513,7 @@ def use_docker():
                         "is not accessible. Please make sure to mount the Docker socket "
                         "/var/run/docker.sock into the container."
                     ),
-                    config.LAMBDA_EXECUTOR,
+                    get_executor_mode(),
                 )
             DO_USE_DOCKER = has_docker
     return DO_USE_DOCKER
@@ -551,7 +553,6 @@ def process_apigateway_invocation(
     query_string_params=None,
     stage_variables=None,
     request_context=None,
-    event_context=None,
 ):
     if path_params is None:
         path_params = {}
@@ -559,8 +560,6 @@ def process_apigateway_invocation(
         stage_variables = {}
     if request_context is None:
         request_context = {}
-    if event_context is None:
-        event_context = {}
     try:
         resource_path = resource_path or path
         event = construct_invocation_event(
@@ -583,7 +582,7 @@ def process_apigateway_invocation(
         inv_result = run_lambda(
             func_arn=func_arn,
             event=event,
-            context=event_context,
+            context=request_context,
             asynchronous=asynchronous,
         )
         return inv_result.result
@@ -806,13 +805,15 @@ def run_lambda(
 
     except Exception as e:
         exc_type, exc_value, exc_traceback = sys.exc_info()
-        # TODO: wrong mapping
         response = {
             "errorType": str(exc_type.__name__),
             "errorMessage": str(e),
             "stackTrace": traceback.format_tb(exc_traceback),
         }
         LOG.info("Error executing Lambda function %s: %s %s", func_arn, e, traceback.format_exc())
+        if isinstance(e, lambda_executors.InvocationException):
+            exc_result = e.result
+            response = run_safe(lambda: json.loads(exc_result)) or response
         log_output = e.log_output if isinstance(e, lambda_executors.InvocationException) else ""
         return InvocationResult(Response(json.dumps(response), status=500), log_output)
     finally:
@@ -2393,7 +2394,7 @@ def on_config_change(config_key: str, config_newvalue: str) -> None:
     )
     LAMBDA_EXECUTOR.cleanup()
     LAMBDA_EXECUTOR = lambda_executors.AVAILABLE_EXECUTORS.get(
-        config_newvalue, lambda_executors.DEFAULT_EXECUTOR
+        get_executor_mode(), lambda_executors.DEFAULT_EXECUTOR
     )
     LAMBDA_EXECUTOR.startup()
 
