@@ -339,6 +339,9 @@ class EC2VPC(GenericBaseModel):
                 )
                 for rt in resp["RouteTables"]:
                     for assoc in rt.get("Associations", []):
+                        # skipping Main association (upstream moto includes default association that cannot be deleted)
+                        if assoc.get("Main"):
+                            continue
                         ec2_client.disassociate_route_table(
                             AssociationId=assoc["RouteTableAssociationId"]
                         )
@@ -425,10 +428,13 @@ class EC2Instance(GenericBaseModel):
         groups = props.get("SecurityGroups", props.get("SecurityGroupIds"))
 
         client = aws_stack.connect_to_service("ec2")
+        kwargs = {}
+        if groups:
+            kwargs["Groups"] = groups
         client.modify_instance_attribute(
-            Groups=groups,
             InstanceId=instance_id,
             InstanceType={"Value": props["InstanceType"]},
+            **kwargs,
         )
         return self._get_state(client)
 
