@@ -21,28 +21,28 @@ from localstack.constants import (
     LS_LOG_TRACE_INTERNAL,
     PATH_USER_REQUEST,
 )
-from localstack.services.cloudwatch.cloudwatch_listener import PATH_GET_RAW_METRICS
+from localstack.http import Router
+from localstack.http.dispatcher import Handler, handler_dispatcher
+from localstack.http.router import RegexConverter
+from localstack.runtime import events
 from localstack.services.generic_proxy import ProxyListener, modify_and_forward, start_proxy_server
 from localstack.services.infra import PROXY_LISTENERS
 from localstack.services.plugins import SERVICE_PLUGINS
 from localstack.services.s3.s3_utils import uses_host_addressing
 from localstack.services.sqs.sqs_listener import is_sqs_queue_url
-from localstack.utils import common, persistence
+from localstack.utils import persistence
 from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_stack import is_internal_call_context, set_default_region_in_headers
 from localstack.utils.aws.request_routing import extract_version_and_action, matches_service_action
-from localstack.utils.common import (
-    TMP_THREADS,
-    empty_context_manager,
-    get_service_protocol,
-    is_port_open,
-    is_root,
-    parse_request_data,
-    run,
-)
-from localstack.utils.common import safe_requests as requests
-from localstack.utils.common import sleep_forever, start_thread, to_bytes, to_str, truncate
+from localstack.utils.functions import empty_context_manager
+from localstack.utils.http import parse_request_data
+from localstack.utils.http import safe_requests as requests
+from localstack.utils.net import is_port_open
+from localstack.utils.run import is_root, run
 from localstack.utils.server.http2_server import HTTPErrorResponse
+from localstack.utils.strings import to_bytes, to_str, truncate
+from localstack.utils.sync import sleep_forever
+from localstack.utils.threads import TMP_THREADS, start_thread
 
 LOG = logging.getLogger(__name__)
 
@@ -73,8 +73,10 @@ class ProxyListenerEdge(ProxyListener):
         self.service_manager = service_manager or SERVICE_PLUGINS
 
     def forward_request(self, method, path, data, headers):
+        # kill the process if we receive this header
+        headers.get(HEADER_KILL_SIGNAL) and sys.exit(0)
 
-        if common.INFRA_STOPPED:
+        if events.infra_stopping.is_set():
             return 503
 
         if config.EDGE_FORWARD_URL:
@@ -82,9 +84,6 @@ class ProxyListenerEdge(ProxyListener):
                 0, method, path, data, headers, target_url=config.EDGE_FORWARD_URL
             )
 
-        # kill the process if we receive this header
-        headers.get(HEADER_KILL_SIGNAL) and sys.exit(0)
-
         target = headers.get("x-amz-target", "")
         auth_header = get_auth_string(method, path, headers, data)
         if auth_header and not headers.get("authorization"):
@@ -267,7 +266,7 @@ def do_forward_request_inmem(api, method, path, data, headers, port=None):
 
 def do_forward_request_network(port, method, path, data, headers, target_url=None):
     # TODO: enable per-service endpoints, to allow deploying in distributed settings
-    target_url = target_url or "%s://%s:%s" % (get_service_protocol(), LOCALHOST, port)
+    target_url = target_url or "%s://%s:%s" % (config.get_protocol(), LOCALHOST, port)
     url = "%s%s" % (target_url, path)
     response = requests.request(
         method, url, data=data, headers=headers, verify=False, stream=True, allow_redirects=False
@@ -451,10 +450,6 @@ def get_api_from_custom_rules(method, path, data, headers):
     if _in_path_or_payload("Action=AssumeRole"):
         return "sts", config.service_port("sts")
 
-    # CloudWatch backdoor API to retrieve raw metrics
-    if path.startswith(PATH_GET_RAW_METRICS):
-        return "cloudwatch", config.service_port("cloudwatch")
-
     # SQS queue requests
     if _in_path_or_payload("QueueUrl=") and _in_path_or_payload("Action="):
         return "sqs", config.service_port("sqs")
@@ -516,6 +511,11 @@ def get_service_port_for_account(service, headers):
 
 
 PROXY_LISTENER_EDGE = ProxyListenerEdge()
+# the ROUTER is part of the edge proxy. use the router to inject custom handlers that are handled before actual
+# service calls
+ROUTER: Router[Handler] = Router(
+    dispatcher=handler_dispatcher(), converters={"regex": RegexConverter}
+)
 
 
 def is_trace_logging_enabled(headers):
@@ -528,21 +528,25 @@ def is_trace_logging_enabled(headers):
 
 
 def do_start_edge(bind_address, port, use_ssl, asynchronous=False):
+    from localstack.http.adapters import RouterListener
     from localstack.services.internal import LocalstackResourceHandler
 
-    # add internal routes as default listener
-    ProxyListener.DEFAULT_LISTENERS.append(LocalstackResourceHandler())
-
     start_dns_server(asynchronous=True)
 
+    listeners = [
+        LocalstackResourceHandler(),  # handle internal resources first
+        RouterListener(ROUTER),  # then custom routes
+        PROXY_LISTENER_EDGE,  # then call the edge proxy listener
+    ]
+
     # get port and start Edge
     print("Starting edge router (http%s port %s)..." % ("s" if use_ssl else "", port))
-    # use use=True here because our proxy allows both, HTTP and HTTPS traffic
+    # use use_ssl=True here because our proxy allows both, HTTP and HTTPS traffic
     proxy = start_proxy_server(
         port,
         bind_address=bind_address,
         use_ssl=True,
-        update_listener=PROXY_LISTENER_EDGE,
+        update_listener=listeners,
         check_port=False,
     )
     if not asynchronous:
