@@ -6,7 +6,7 @@ import time
 from typing import Dict
 
 from localstack import config
-from localstack.constants import ENV_INTERNAL_TEST_RUN, MOTO_ACCOUNT_ID, TEST_AWS_ACCOUNT_ID
+from localstack.constants import MOTO_ACCOUNT_ID, TEST_AWS_ACCOUNT_ID
 from localstack.services.events.scheduler import JobScheduler
 from localstack.services.generic_proxy import ProxyListener, RegionBackend
 from localstack.utils.aws import aws_stack
@@ -129,7 +129,7 @@ def handle_put_rule(data: Dict):
     if schedule:
         job_func = get_scheduled_rule_func(data)
         cron = convert_schedule_to_cron(schedule)
-        LOG.debug("Adding new scheduled Events rule with cron schedule %s" % cron)
+        LOG.debug("Adding new scheduled Events rule with cron schedule %s", cron)
 
         job_id = JobScheduler.instance().add_job(job_func, cron, enabled)
         rule_scheduled_jobs = EventsBackend.get().rule_scheduled_jobs
@@ -176,7 +176,7 @@ class ProxyListenerEvents(ProxyListener):
 
             elif action == "PutEvents":
                 # keep track of events for local integration testing
-                if os.environ.get(ENV_INTERNAL_TEST_RUN):
+                if config.is_local_test_mode():
                     TEST_EVENTS_CACHE.extend(parsed_data.get("Entries", []))
 
         return True
